@@ -65,7 +65,6 @@ import org.apache.uima.util.InvalidXMLException;
 import org.apache.uima.util.Level;
 import org.apache.uima.util.Logger;
 import org.apache.uima.util.ProcessTrace;
-import org.apache.uima.util.ProcessTraceEvent;
 import org.apache.uima.util.impl.ProcessTraceEvent_impl;
 
 /**
@@ -219,7 +218,7 @@ public class AggregateAnalysisEngine_impl extends AnalysisEngineImplBase impleme
       // now iterate over components and call their setResultSpecification methods
       Iterator<AnalysisEngine> componentIter = _getASB().getComponentAnalysisEngines().values().iterator();
       while (componentIter.hasNext()) {
-        AnalysisEngine ae = (AnalysisEngine) componentIter.next();
+        AnalysisEngine ae = componentIter.next();
         ae.setResultSpecification(resultSpecForComponents);
       }
     }
@@ -290,7 +289,7 @@ public class AggregateAnalysisEngine_impl extends AnalysisEngineImplBase impleme
     Map<String, AnalysisEngine> components = this._getASB().getComponentAnalysisEngines();
     Iterator<AnalysisEngine> it = components.values().iterator();
     while (it.hasNext()) {
-      ConfigurableResource component = (ConfigurableResource) it.next();
+      ConfigurableResource component = it.next();
       component.reconfigure();
     }
     //and the FlowController
@@ -332,7 +331,7 @@ public class AggregateAnalysisEngine_impl extends AnalysisEngineImplBase impleme
       //call components in the order specified in the flow
       if (orderedNodes != null) {
         for (int i = 0; i < orderedNodes.length; i++) {
-          AnalysisEngine component = (AnalysisEngine)components.remove(orderedNodes[i]);  
+          AnalysisEngine component = components.remove(orderedNodes[i]);  
           component.collectionProcessComplete();
         }
       }
@@ -539,7 +538,7 @@ public class AggregateAnalysisEngine_impl extends AnalysisEngineImplBase impleme
   }
 
   /**
-   * Returns the default FlowController declaration to be used if the aggregate AE descriptor does
+   * @return the default FlowController declaration to be used if the aggregate AE descriptor does
    * not specify one.
    */
   protected FlowControllerDeclaration getDefaultFlowControllerDeclaration() {
@@ -610,7 +609,7 @@ public class AggregateAnalysisEngine_impl extends AnalysisEngineImplBase impleme
         if (ae instanceof AnalysisEngineImplBase) {
           ProcessTrace subPT = ((AnalysisEngineImplBase) ae).buildProcessTraceFromMBeanStats();
           if (subPT.getEvents().size() > 0) {
-            procEvt.addSubEvent((ProcessTraceEvent) subPT.getEvents().get(0));
+            procEvt.addSubEvent(subPT.getEvents().get(0));
           }
         }
       }
