@@ -66,7 +66,7 @@ import org.apache.uima.util.impl.ProcessTrace_impl;
  */
 public abstract class AnalysisEngineImplBase extends ConfigurableResource_ImplBase implements
         TextAnalysisEngine {
-
+  
   /* (non-Javadoc)
    * @see org.apache.uima.resource.Resource_ImplBase#setMetaData(org.apache.uima.resource.metadata.ResourceMetaData)
    */
@@ -260,7 +260,9 @@ public abstract class AnalysisEngineImplBase extends ConfigurableResource_ImplBa
           throws ResultNotSupportedException, AnalysisEngineProcessException {
     setResultSpecification(aResultSpec);
     process(aCAS);
-    buildProcessTraceFromMBeanStats(aTrace);
+    if (isProcessTraceEnabled()) {  // a slight performance speedup https://issues.apache.org/jira/browse/UIMA-4151
+      buildProcessTraceFromMBeanStats(aTrace); 
+    }
   }
 
   public ProcessTrace process(CAS aCAS) throws AnalysisEngineProcessException {
@@ -270,7 +272,9 @@ public abstract class AnalysisEngineImplBase extends ConfigurableResource_ImplBa
       CAS cas = iter.next();
       cas.release();
     }
-    return buildProcessTraceFromMBeanStats();
+    // https://issues.apache.org/jira/browse/UIMA-4151
+    return isProcessTraceEnabled() ? buildProcessTraceFromMBeanStats() : ProcessTrace_impl.disabledProcessTrace;
+//    return buildProcessTraceFromMBeanStats();
   }
 
   /**
@@ -485,6 +489,7 @@ public abstract class AnalysisEngineImplBase extends ConfigurableResource_ImplBa
    * access it.
    * 
    * @see org.apache.uima.resource.ConfigurationManager#getCurrentConfigParameterSettings(String)
+   * @return -
    */
   protected ConfigurationParameterSettings getCurrentConfigParameterSettings() {
     return getUimaContextAdmin().getConfigurationManager().getCurrentConfigParameterSettings(
@@ -492,7 +497,7 @@ public abstract class AnalysisEngineImplBase extends ConfigurableResource_ImplBa
   }
 
   /**
-   * Gets the MBean that provides the management interface to this AE. Returns the same object as
+   * @return the MBean that provides the management interface to this AE. Returns the same object as
    * getManagementInterface() but casted to the AnalysisEngineManagement type.
    */
   protected AnalysisEngineManagementImpl getMBean() {
@@ -527,7 +532,7 @@ public abstract class AnalysisEngineImplBase extends ConfigurableResource_ImplBa
   }
 
   /**
-   * Construct a ProcessTrace object that represents the last execution of this AnalysisEngine. This
+   * @return a constructed ProcessTrace object that represents the last execution of this AnalysisEngine. This
    * is used so that we can return a ProcessTrace object from each process() call for backwards
    * compatibility with version 1.x.
    */
@@ -541,6 +546,7 @@ public abstract class AnalysisEngineImplBase extends ConfigurableResource_ImplBa
    * Modify an existing ProcessTrace object by adding events that represent the last excecution of
    * this AnalysisEngine. This is used so that we can return a ProcessTrace object from each
    * process() call for backwards compatibility with version 1.x.
+   * @param trace -
    */
   protected void buildProcessTraceFromMBeanStats(ProcessTrace trace) {
     // this is the implementation for primitives only. Aggregate AE overrides.
