@@ -46,7 +46,6 @@ import org.apache.uima.resource.ResourceConfigurationException;
 import org.apache.uima.resource.ResourceCreationSpecifier;
 import org.apache.uima.resource.ResourceInitializationException;
 import org.apache.uima.resource.ResourceSpecifier;
-import org.apache.uima.resource.impl.ResourceCreationSpecifier_impl;
 import org.apache.uima.resource.metadata.ProcessingResourceMetaData;
 import org.apache.uima.resource.metadata.ResourceMetaData;
 import org.apache.uima.util.Level;
@@ -279,7 +278,7 @@ public class PrimitiveAnalysisEngine_impl extends AnalysisEngineImplBase impleme
   public void destroy() {
     if (mAnalysisComponent != null) {
       mAnalysisComponent.destroy();
-      getLogger().logrb(Level.CONFIG, CLASS_NAME.getName(), "initialize", LOG_RESOURCE_BUNDLE,
+      getLogger().logrb(Level.CONFIG, CLASS_NAME.getName(), "destroy", LOG_RESOURCE_BUNDLE,
               "UIMA_analysis_engine_destroyed__CONFIG", getMetaData().getName());
     }
     super.destroy();
@@ -339,6 +338,7 @@ public class PrimitiveAnalysisEngine_impl extends AnalysisEngineImplBase impleme
    * 
    * @param aCAS
    *          CAS to be processed by annotator
+   * @throws AnalysisEngineProcessException -         
    */
   protected void callAnalysisComponentProcess(CAS aCAS) throws AnalysisEngineProcessException {
     // logging and instrumentation
@@ -503,6 +503,8 @@ public class PrimitiveAnalysisEngine_impl extends AnalysisEngineImplBase impleme
    * Calls the Analysis Component's next() method.
    * 
    * @return CAS returned by the analysis component
+   * @throws AnalysisEngineProcessException -
+   * @throws ResultNotSupportedException -
    */
   protected CAS callAnalysisComponentNext() throws AnalysisEngineProcessException,
           ResultNotSupportedException {
