@@ -173,7 +173,7 @@ public final class ResultSpecification_impl extends MetaDataObject_impl implemen
    *   if the type system is available.  Otherwise, the type system *must*
    *   be set via a method call prior to querying the result spec, with the
    *   one exception of the method getResultTypesAndFeaturesWithoutCompiling
-   * @param aTypeSystem
+   * @param aTypeSystem -
    */
   public ResultSpecification_impl(TypeSystem aTypeSystem) {
     this();
@@ -213,7 +213,7 @@ public final class ResultSpecification_impl extends MetaDataObject_impl implemen
   
   private TypeOrFeature[] getResultTypesAndFeatures(boolean skipLanguageFilter, String language) {
     List<TypeOrFeature> r = new ArrayList<TypeOrFeature>();
-    if (rsTypesMap.nbrOfTypes() == 0 && needsCompilation == false) {
+    if (rsTypesMap.nbrOfTypes() == 0 && !needsCompilation) {
       // being called on results of intersection
       // probably by a test case, not a normal call
       // attempt to construct a plausible representation
@@ -551,7 +551,7 @@ public final class ResultSpecification_impl extends MetaDataObject_impl implemen
   public String toString() {
     StringBuilder sb = new StringBuilder();
     sb.append("org.apache.uima.analysis_engine.impl.ResultSpecification_impl:\n");
-    sb.append("  needsCompilation = ").append(needsCompilation).append("\n");
+    sb.append("  needsCompilation = ").append(needsCompilation).append('\n');
 //    sb.append("lang2int = ").append(lang2int).append("\n");
 //    sb.append("name2tof_langs = ").append(name2tof_langs).append("\n");
 //    sb.append("withSubtypesName2tof_langs = ").append(withSubtypesName2tof_langs).append("\n");
@@ -570,10 +570,10 @@ public final class ResultSpecification_impl extends MetaDataObject_impl implemen
         Entry<String, RsLangs> e = (Entry<String, RsLangs>) o;
         String k = e.getKey();
         k = k + "        ".substring(k.length()%8);
-        sb.append(" key: ").append(k).append("  value: ").append(e.getValue()).append("\n");
+        sb.append(" key: ").append(k).append("  value: ").append(e.getValue()).append('\n');
       }
     }
-    sb.append("\n\nmTypeSystem = ").append(mTypeSystem).append("\n");
+    sb.append("\n\nmTypeSystem = ").append(mTypeSystem).append('\n');
     return sb.toString();
   }
 
