@@ -472,9 +472,10 @@ public interface CAS extends AbstractCas {
    * 
    * @param type
    *          The type of the FS.
+   * @param <T> the Java cover class for the FS being created
    * @return The new FS.
    */
-  FeatureStructure createFS(Type type) throws CASRuntimeException;
+  <T extends FeatureStructure> T createFS(Type type) throws CASRuntimeException;
 
   /**
    * Create a new feature structure array.
@@ -704,20 +705,23 @@ public interface CAS extends AbstractCas {
   /**
    * Get the standard annotation index.
    * 
+   * @param <T> either Annotation (if JCas is in use) or AnnotationImpl 
+   * 
    * @return The standard annotation index.
    */
-  AnnotationIndex<AnnotationFS> getAnnotationIndex();
+  <T extends AnnotationFS> AnnotationIndex<T> getAnnotationIndex();
 
   /**
    * Get the standard annotation index restricted to a specific annotation type.
    * 
    * @param type
    *          The annotation type the index is restricted to.
+   * @param <T> the topmost Java class corresponding to the type
    * @return The standard annotation index, restricted to <code>type</code>.
    * @exception CASRuntimeException When <code>type</code> is not an annotation type.
    */
-  AnnotationIndex<AnnotationFS> getAnnotationIndex(Type type) throws CASRuntimeException;
-
+  <T extends AnnotationFS> AnnotationIndex<T> getAnnotationIndex(Type type) throws CASRuntimeException;
+  
   /**
    * Create a new annotation. Note that you still need to insert the annotation into the index
    * repository yourself.
@@ -728,17 +732,20 @@ public interface CAS extends AbstractCas {
    *          The start of the annotation.
    * @param end
    *          The end of the annotation.
+   * @param <T> the Java class corresponding to the type
    * @return A new annotation object.
    */
-  AnnotationFS createAnnotation(Type type, int begin, int end);
+  <T extends AnnotationFS> AnnotationFS createAnnotation(Type type, int begin, int end);
 
   /**
    * Get the document annotation. The document has a string-valued feature called "language" where
    * the document language is specified.
    * 
-   * @return The document annotation, or <code>null</code> if there is none.
+   * @param <T> the Java class for the document annotation.  Could be the JCas cover class or FeatureStructure
+   * @return The document annotation, or <code>null</code> if there is none.  The return value is the
+   *         JCas cover class or the plain Java cover class for FeatureStructures if JCas is not in use.
    */
-  AnnotationFS getDocumentAnnotation();
+  <T extends AnnotationFS> T getDocumentAnnotation();
 
   /**
    * Informs the CAS of relevant information about the component that is currently processing it.
@@ -1001,7 +1008,11 @@ public interface CAS extends AbstractCas {
 
   /**
    * Remove a feature structure from all indexes in the repository associated with this CAS View.
+   * The remove operation removes the exact fs from the indexes, unlike operations such as 
+   * moveTo which use the fs argument as a template.  
    * 
+   * It is not an error if the FS is not present in the indexes. 
+   *
    * @param fs
    *          The Feature Structure to be removed.
    * @exception NullPointerException
