@@ -29,13 +29,16 @@ import org.apache.uima.cas.text.AnnotationTree;
 
 /**
  * Implementation of annotation indexes.
+ * A wrapper not an extend, because 
+ *   the constructor argument index has 
+ *     customization for additional fns and
+ *     a ref to it's enclosing instance of the FSIndexRepositoryImpl.
  */
-public class AnnotationIndexImpl<T extends AnnotationFS> implements AnnotationIndex<T> {
-
-  private FSIndex<AnnotationFS> index;
+public class AnnotationIndexImpl <T extends AnnotationFS> implements AnnotationIndex<T> {
 
+  private FSIndex<T> index;
   
-  public AnnotationIndexImpl(FSIndex<AnnotationFS> index) {
+  public AnnotationIndexImpl(FSIndex<T> index) {
     super();
     this.index = index;
   }
@@ -45,6 +48,7 @@ public class AnnotationIndexImpl<T extends AnnotationFS> implements AnnotationIn
    * 
    * @see org.apache.uima.cas.FSIndex#size()
    */
+  @Override
   public int size() {
     return this.index.size();
   }
@@ -54,6 +58,7 @@ public class AnnotationIndexImpl<T extends AnnotationFS> implements AnnotationIn
    * 
    * @see org.apache.uima.cas.FSIndex#getType()
    */
+  @Override
   public Type getType() {
     return this.index.getType();
   }
@@ -63,6 +68,7 @@ public class AnnotationIndexImpl<T extends AnnotationFS> implements AnnotationIn
    * 
    * @see org.apache.uima.cas.FSIndex#contains(org.apache.uima.cas.FeatureStructure)
    */
+  @Override
   public boolean contains(FeatureStructure fs) {
     return this.index.contains(fs);
   }
@@ -72,6 +78,7 @@ public class AnnotationIndexImpl<T extends AnnotationFS> implements AnnotationIn
    * 
    * @see org.apache.uima.cas.FSIndex#find(org.apache.uima.cas.FeatureStructure)
    */
+  @Override
   public FeatureStructure find(FeatureStructure fs) {
     return this.index.find(fs);
   }
@@ -82,6 +89,7 @@ public class AnnotationIndexImpl<T extends AnnotationFS> implements AnnotationIn
    * @see org.apache.uima.cas.FSIndex#compare(org.apache.uima.cas.FeatureStructure,
    *      org.apache.uima.cas.FeatureStructure)
    */
+  @Override
   public int compare(FeatureStructure fs1, FeatureStructure fs2) {
     return this.index.compare(fs1, fs2);
   }
@@ -91,6 +99,7 @@ public class AnnotationIndexImpl<T extends AnnotationFS> implements AnnotationIn
    * 
    * @see org.apache.uima.cas.FSIndex#iterator()
    */
+  @Override
   public FSIterator<T> iterator() {
     return (FSIterator<T>) this.index.iterator();
   }
@@ -100,6 +109,7 @@ public class AnnotationIndexImpl<T extends AnnotationFS> implements AnnotationIn
    * 
    * @see org.apache.uima.cas.FSIndex#iterator(org.apache.uima.cas.FeatureStructure)
    */
+  @Override
   public FSIterator<T> iterator(FeatureStructure fs) {
     return (FSIterator<T>) this.index.iterator(fs);
   }
@@ -109,6 +119,7 @@ public class AnnotationIndexImpl<T extends AnnotationFS> implements AnnotationIn
    * 
    * @see org.apache.uima.cas.FSIndex#getIndexingStrategy()
    */
+  @Override
   public int getIndexingStrategy() {
     return this.index.getIndexingStrategy();
   }
@@ -118,11 +129,15 @@ public class AnnotationIndexImpl<T extends AnnotationFS> implements AnnotationIn
    * 
    * @see org.apache.uima.cas.text.AnnotationIndex#iterator(boolean)
    */
+  @Override
   public FSIterator<T> iterator(boolean ambiguous) {
     if (ambiguous) {
       return (FSIterator<T>) this.index.iterator();
     }
-    return new Subiterator<T>((FSIterator<T>) this.index.iterator());
+    // return non-constrained, non-strict, unambiguous iterator
+    boolean strict = true;
+    boolean isBounded = false;
+    return new Subiterator<T>(this.index.iterator(), null, 0, 0, ambiguous, strict, isBounded, ((FSIndexRepositoryImpl.IndexImpl<T>)(this.index)).getFsRepositoryImpl());
   }
 
   /*
@@ -130,6 +145,7 @@ public class AnnotationIndexImpl<T extends AnnotationFS> implements AnnotationIn
    * 
    * @see org.apache.uima.cas.text.AnnotationIndex#subiterator(org.apache.uima.cas.text.AnnotationFS)
    */
+  @Override
   public FSIterator<T> subiterator(AnnotationFS annot) {
     return subiterator(annot, true, true);
   }
@@ -140,8 +156,10 @@ public class AnnotationIndexImpl<T extends AnnotationFS> implements AnnotationIn
    * @see org.apache.uima.cas.text.AnnotationIndex#subiterator(org.apache.uima.cas.text.AnnotationFS,
    *      boolean, boolean)
    */
+  @Override
   public FSIterator<T> subiterator(AnnotationFS annot, boolean ambiguous, boolean strict) {
-    return new Subiterator<T>((FSIterator<T>)this.index.iterator(), annot, ambiguous, strict);
+    boolean isBounded = true;
+    return new Subiterator<T>(this.index.iterator(), annot, 0, 0, ambiguous, strict, isBounded, ((FSIndexRepositoryImpl.IndexImpl<T>)(this.index)).getFsRepositoryImpl());
   }
 
   /*
@@ -149,6 +167,7 @@ public class AnnotationIndexImpl<T extends AnnotationFS> implements AnnotationIn
    * 
    * @see org.apache.uima.cas.text.AnnotationIndex#tree(org.apache.uima.cas.text.AnnotationFS)
    */
+  @Override
   public AnnotationTree<T> tree(T annot) {
     AnnotationTreeImpl<T> tree = new AnnotationTreeImpl<T>();
     AnnotationTreeNodeImpl<T> root = new AnnotationTreeNodeImpl<T>();
@@ -172,8 +191,8 @@ public class AnnotationIndexImpl<T extends AnnotationFS> implements AnnotationIn
   }
 
   @Override
-  public FSIndex withSnapshotIterators() {
-    return new AnnotationIndexImpl(index.withSnapshotIterators());
+  public FSIndex<T> withSnapshotIterators() {
+    return new AnnotationIndexImpl<T>(index.withSnapshotIterators());
   }
-
+  
 }
