@@ -248,9 +248,10 @@ public interface LowLevelCAS {
    * 
    * @param fsRef
    *          The FS reference.
+   * @param <T> the Java class for the Feature Structure
    * @return A FS object corresponding to the input reference.
    */
-  FeatureStructure ll_getFSForRef(int fsRef);
+  <T extends FeatureStructure> T ll_getFSForRef(int fsRef);
 
   /**
    * Get the value of an integer valued feature.
