@@ -71,6 +71,8 @@ import org.apache.uima.resource.ResourceInitializationException;
 public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
   
   private final static int[] INT0 = new int[0];
+  
+  private final static IntVector zeroLengthIntVector = new IntVector(1);  // capacity is 1; 0 makes length default to 16
 
   private static class ListIterator<T> implements Iterator<T> {
 
@@ -85,10 +87,12 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
       this.len = (max < list.size()) ? max : list.size();
     }
 
+    @Override
     public boolean hasNext() {
       return this.pos < this.len;
     }
 
+    @Override
     public T next() {
       if (!hasNext()) {
         throw new NoSuchElementException();
@@ -98,6 +102,7 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
       return o;
     }
 
+    @Override
     public void remove() {
       throw new UnsupportedOperationException();
     }
@@ -110,9 +115,9 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
 
   // static maps ok for now - only built-in mappings stored here
   // which are the same for all type system instances
-  private final static Map<String, String> arrayComponentTypeNameMap = new HashMap<String, String>();
+  private final static Map<String, String> arrayComponentTypeNameMap = new HashMap<String, String>(9);
 
-  private final static Map<String, String> arrayTypeComponentNameMap = new HashMap<String, String>();
+  private final static Map<String, String> arrayTypeComponentNameMap = new HashMap<String, String>(9);
 
   private static final String arrayTypeSuffix = "[]";
 
@@ -158,29 +163,42 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
   // feature is declared on).
   private final StringToIntMap featureMap;
 
+  /**
+   * List indexed by supertype code
+   * Value is an IntVector representing type codes for directly subsumed types (just one level below)
+   */
   private final List<IntVector> tree; // Collection of IntVectors encoding type tree
 
+  /**
+   * List indexed by supertype code, 
+   * Value is bits set of subtypes of that code, including the type itself
+   */
   private final List<BitSet> subsumes; // Collection of BitSets for subsumption relation
-
+  
   private final IntVector intro;
 
   // Indicates which type introduces a feature (domain)
   private final IntVector featRange; // Indicates range type of features
-
-  private final ArrayList<IntVector> approp; // For each type, an IntVector of appropriate
-
-  // features
+  
+  /**
+   * For each type, an IntVector of appropriate feature codes
+   */
+  private final ArrayList<IntVector> approp; // For each type, an IntVector of appropriate features
 
   // Code of root of hierarchy (will be 1 with current implementation)
-  private int top;
+  private static final int top = 1;
 
-  // An ArrayList (unsynchronized) of TypeImpl objects.
+  /**
+   * An ArrayList, unsynchronized, indexed by typeCode, of Type objects
+   */
   private final List<Type> types;
 
   // An ArrayList (unsynchronized) of FeatureImpl objects.
   private final List<Feature> features;
 
-  // List of parent types.
+  /**
+   * array of parent typeCodes indexed by typeCode
+   */
   private final IntVector parents;
 
   // String sets for string subtypes.
@@ -188,17 +206,17 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
 
   // This map contains an entry for every subtype of the string type. The value is a pointer into
   // the stringSets array list.
-  private final IntRedBlackTree stringSetMap;
+  private final IntRedBlackTree stringSetMap = new IntRedBlackTree();
 
   // For each type, remember if an array of this component type has already
   // been created.
-  private final IntRedBlackTree componentToArrayTypeMap;
+  private final IntRedBlackTree componentToArrayTypeMap = new IntRedBlackTree();
 
   // A mapping from array types to their component types.
-  private final IntRedBlackTree arrayToComponentTypeMap;
+  private final IntRedBlackTree arrayToComponentTypeMap = new IntRedBlackTree();
 
   // A mapping from array type codes to array type objects.
-  private final RedBlackTree<TypeImpl> arrayCodeToTypeMap;
+  private final RedBlackTree<TypeImpl> arrayCodeToTypeMap = new RedBlackTree<>();
 
   // Is the type system locked?
   private boolean locked = false;
@@ -273,67 +291,67 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
   TypeImpl doubleArrayType;
 
   // int topTypeCode;
-  int intTypeCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
+  static final int intTypeCode = 2;
 
-  int stringTypeCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
+  static final int stringTypeCode = 4;
 
-  int floatTypeCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
+  static final int floatTypeCode = 3;
 
-  int arrayBaseTypeCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
+  static final int arrayBaseTypeCode = 5;
 
-  public int intArrayTypeCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
+  public static final int intArrayTypeCode = 8;
 
-  public int floatArrayTypeCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
+  public static final int floatArrayTypeCode = 7;
 
-  public int stringArrayTypeCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
+  public static final int stringArrayTypeCode = 9;
 
-  public int fsArrayTypeCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
+  public static final int fsArrayTypeCode = 6;
 
-  int sofaTypeCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
+  static final int sofaTypeCode = 33;
 
-  int annotTypeCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
+  static final int annotTypeCode = 35;
 
-  int annotBaseTypeCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
+  static final int annotBaseTypeCode = 34;
 
-  int byteTypeCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
+  static final int byteTypeCode = 24;
 
-  int booleanTypeCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
+  static final int booleanTypeCode = 23;
 
-  int shortTypeCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
+  static final int shortTypeCode = 25;
 
-  int longTypeCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
+  static final int longTypeCode = 26;
 
-  int doubleTypeCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
+  static final int doubleTypeCode = 27;
 
-  public int byteArrayTypeCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
+  public static final int byteArrayTypeCode = 29;
 
-  public int booleanArrayTypeCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
+  public static final int booleanArrayTypeCode = 28;
 
-  public int shortArrayTypeCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
+  public static final int shortArrayTypeCode = 30;
 
-  public int longArrayTypeCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
+  public static final int longArrayTypeCode = 31;
 
-  public int doubleArrayTypeCode = LowLevelTypeSystem.UNKNOWN_TYPE_CODE;
+  public static final int doubleArrayTypeCode = 32;
 
-  public int sofaNumFeatCode = LowLevelTypeSystem.UNKNOWN_FEATURE_CODE;  // ref from another pkg
+  public static final int sofaNumFeatCode = 9;  // ref from another pkg
 
-  public int sofaIdFeatCode = LowLevelTypeSystem.UNKNOWN_FEATURE_CODE;
+  public static final int sofaIdFeatCode = 10;
   
-  public int sofaStringFeatCode = LowLevelTypeSystem.UNKNOWN_FEATURE_CODE;
+  public static final int sofaStringFeatCode = 13;
 
-  int sofaMimeFeatCode = LowLevelTypeSystem.UNKNOWN_FEATURE_CODE;
+  static final int sofaMimeFeatCode = 11;
 
-  int sofaUriFeatCode = LowLevelTypeSystem.UNKNOWN_FEATURE_CODE;
+  static final int sofaUriFeatCode = 14;
 
-  int sofaArrayFeatCode = LowLevelTypeSystem.UNKNOWN_FEATURE_CODE;
+  static final int sofaArrayFeatCode = 12;
 
-  public int annotSofaFeatCode = LowLevelTypeSystem.UNKNOWN_FEATURE_CODE; // ref from another pkg
+  public static final int annotSofaFeatCode = 15; // ref from another pkg
 
-  public int startFeatCode = LowLevelTypeSystem.UNKNOWN_FEATURE_CODE;
+  public static final int startFeatCode = 16;
 
-  public int endFeatCode = LowLevelTypeSystem.UNKNOWN_FEATURE_CODE;
+  public static final int endFeatCode = 17;
 
-  int langFeatCode = LowLevelTypeSystem.UNKNOWN_FEATURE_CODE;
+  static final int langFeatCode = 18;
 
   /**
    * Default constructor.
@@ -370,10 +388,6 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
     this.features = new ArrayList<Feature>();
     this.features.add(null);
     this.stringSets = new ArrayList<String[]>();
-    this.stringSetMap = new IntRedBlackTree();
-    this.componentToArrayTypeMap = new IntRedBlackTree();
-    this.arrayToComponentTypeMap = new IntRedBlackTree();
-    this.arrayCodeToTypeMap = new RedBlackTree<TypeImpl>();
     this.parents = new IntVector();
     this.parents.add(0);
 
@@ -448,26 +462,26 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
 //    this.arrayBaseTypeCode = this.arrayBaseType.getCode();
 
     final Type sofaT = this.sofaType;
-    this.sofaNumFeatCode = ll_getCodeForFeature(sofaT
-        .getFeatureByBaseName(CAS.FEATURE_BASE_NAME_SOFANUM));
-    this.sofaIdFeatCode = ll_getCodeForFeature(sofaT
-        .getFeatureByBaseName(CAS.FEATURE_BASE_NAME_SOFAID));
-    this.sofaMimeFeatCode = ll_getCodeForFeature(sofaT
-        .getFeatureByBaseName(CAS.FEATURE_BASE_NAME_SOFAMIME));
-    this.sofaUriFeatCode = ll_getCodeForFeature(sofaT
-        .getFeatureByBaseName(CAS.FEATURE_BASE_NAME_SOFAURI));
-    this.sofaArrayFeatCode = ll_getCodeForFeature(sofaT
-        .getFeatureByBaseName(CAS.FEATURE_BASE_NAME_SOFAARRAY));
-    this.annotSofaFeatCode = ll_getCodeForFeature(this.annotBaseType
-        .getFeatureByBaseName(CAS.FEATURE_BASE_NAME_SOFA));
-    this.startFeatCode = ll_getCodeForFeature(this.annotType
-        .getFeatureByBaseName(CAS.FEATURE_BASE_NAME_BEGIN));
-    this.endFeatCode = ll_getCodeForFeature(this.annotType
-        .getFeatureByBaseName(CAS.FEATURE_BASE_NAME_END));
-    this.langFeatCode = ll_getCodeForFeature(this.docType
-        .getFeatureByBaseName(CAS.FEATURE_BASE_NAME_LANGUAGE));
-    this.sofaStringFeatCode = ll_getCodeForFeature(sofaT
-        .getFeatureByBaseName(CAS.FEATURE_BASE_NAME_SOFASTRING));
+    if (sofaNumFeatCode != ll_getCodeForFeature(sofaT
+        .getFeatureByBaseName(CAS.FEATURE_BASE_NAME_SOFANUM))) throw new RuntimeException();
+    if (sofaIdFeatCode != ll_getCodeForFeature(sofaT
+        .getFeatureByBaseName(CAS.FEATURE_BASE_NAME_SOFAID))) throw new RuntimeException();
+    if (sofaMimeFeatCode != ll_getCodeForFeature(sofaT
+        .getFeatureByBaseName(CAS.FEATURE_BASE_NAME_SOFAMIME))) throw new RuntimeException();
+    if (sofaUriFeatCode != ll_getCodeForFeature(sofaT
+        .getFeatureByBaseName(CAS.FEATURE_BASE_NAME_SOFAURI))) throw new RuntimeException();
+    if (sofaArrayFeatCode != ll_getCodeForFeature(sofaT
+        .getFeatureByBaseName(CAS.FEATURE_BASE_NAME_SOFAARRAY))) throw new RuntimeException();
+    if (annotSofaFeatCode != ll_getCodeForFeature(this.annotBaseType
+        .getFeatureByBaseName(CAS.FEATURE_BASE_NAME_SOFA))) throw new RuntimeException();
+    if (startFeatCode != ll_getCodeForFeature(this.annotType
+        .getFeatureByBaseName(CAS.FEATURE_BASE_NAME_BEGIN))) throw new RuntimeException();
+    if (endFeatCode != ll_getCodeForFeature(this.annotType
+        .getFeatureByBaseName(CAS.FEATURE_BASE_NAME_END))) throw new RuntimeException();
+    if (langFeatCode != ll_getCodeForFeature(this.docType
+        .getFeatureByBaseName(CAS.FEATURE_BASE_NAME_LANGUAGE))) throw new RuntimeException();
+    if (sofaStringFeatCode != ll_getCodeForFeature(sofaT
+        .getFeatureByBaseName(CAS.FEATURE_BASE_NAME_SOFASTRING))) throw new RuntimeException();
   }
 
   // Some implementation helpers for users of the type system.
@@ -497,25 +511,27 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
     return feats;
   }
 
+  @Override
   public Type getParent(Type t) {
     return ((TypeImpl) t).getSuperType();
   }
 
+  @Override
   public int ll_getParentType(int typeCode) {
     return this.parents.get(typeCode);
   }
 
   int ll_computeArrayParentFromComponentType(int componentType) {
     if (ll_isPrimitiveType(componentType) ||
-    // note: not using this.top - until we can confirm this is set
+    // note: not using top - until we can confirm this is set
         // in all cases
         (ll_getTypeForCode(componentType).getName().equals(CAS.TYPE_NAME_TOP))) {
-      return this.arrayBaseTypeCode;
+      return arrayBaseTypeCode;
     }
     // is a subtype of FSArray.
     // note: not using this.fsArray - until we can confirm this is set in
     // all cases
-    return this.fsArrayTypeCode;
+    return fsArrayTypeCode;
     // return ll_getArrayType(ll_getParentType(componentType));
   }
 
@@ -544,6 +560,7 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
    *          The name of the type.
    * @return A type object, or <code>null</code> if no such type exists.
    */
+  @Override
   public Type getType(String typeName) {
     final int typeCode = ll_getCodeForTypeName(typeName);
     if (typeCode < LEAST_TYPE_CODE) {
@@ -570,6 +587,7 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
    *          The name of the feature.
    * @return An feature object, or <code>null</code> if no such feature exists.
    */
+  @Override
   public Feature getFeatureByFullName(String featureName) {
     // if (!this.featureMap.containsKey(featureName)) {
     // return null;
@@ -616,6 +634,7 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
    *          The type node under which the new type should be attached.
    * @return The new type, or <code>null</code> if <code>typeName</code> is already in use.
    */
+  @Override
   public Type addType(String typeName, Type mother) throws CASAdminException {
     if (this.locked) {
       throw new CASAdminException(CASAdminException.TYPE_SYSTEM_LOCKED);
@@ -703,6 +722,7 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
     return type;
   }
 
+  @Override
   public Feature addFeature(String featureName, Type domainType, Type rangeType)
       throws CASAdminException {
     return addFeature(featureName, domainType, rangeType, true);
@@ -711,6 +731,7 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
   /**
    * @see TypeSystemMgr#addFeature(String, Type, Type)
    */
+  @Override
   public Feature addFeature(String featureName, Type domainType, Type rangeType,
       boolean multipleReferencesAllowed) throws CASAdminException {
     // assert(featureName != null);
@@ -753,6 +774,7 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
    * 
    * @return The iterator.
    */
+  @Override
   public Iterator<Type> getTypeIterator() {
     Iterator<Type> it = new ListIterator<Type>(this.types, this.numCommittedTypes);
     // The first element is null, so skip it.
@@ -760,6 +782,7 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
     return it;
   }
 
+  @Override
   public Iterator<Feature> getFeatures() {
     Iterator<Feature> it = this.features.iterator();
     // The first element is null, so skip it.
@@ -772,8 +795,9 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
    * 
    * @return The top type.
    */
+  @Override
   public Type getTopType() {
-    return this.types.get(this.top);
+    return this.types.get(top);
   }
 
   /**
@@ -784,6 +808,7 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
    *          Input type.
    * @return The list of types subsumed by <code>type</code>.
    */
+  @Override
   public List<Type> getProperlySubsumedTypes(Type type) {
     List<Type> subList = new ArrayList<Type>();
     Iterator<Type> typeIt = getTypeIterator();
@@ -804,23 +829,86 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
    *          The input type.
    * @return A vector of the directly subsumed types.
    */
+  @Override
   public Vector<Type> getDirectlySubsumedTypes(Type type) {
     return new Vector<Type>(getDirectSubtypes(type));
   }
 
+  @Override
   public List<Type> getDirectSubtypes(Type type) {
     if (type.isArray()) {
       return new ArrayList<Type>();
     }
-    List<Type> list = new ArrayList<Type>();
     IntVector sub = this.tree.get(((TypeImpl) type).getCode());
     final int max = sub.size();
+    List<Type> list = new ArrayList<Type>(max);
+    
     for (int i = 0; i < max; i++) {
       list.add(this.types.get(sub.get(i)));
     }
     return list;
   }
+  
+  /**
+   * 
+   * @param type whose direct instantiable subtypes to iterate over
+   * @return an iterator over the direct instantiable subtypes
+   */
+  public Iterator<Type> getDirectSubtypesIterator(final Type type) {
+       
+    return new Iterator<Type>() {
 
+      private IntVector sub = (type.isArray()) ? zeroLengthIntVector :  TypeSystemImpl.this.tree.get(((TypeImpl) type).getCode());
+      private int pos = 0; 
+      
+      private boolean isTop = (((TypeImpl)type).getCode() == top);
+       
+      {
+        if (isTop) {
+          skipOverNonCreatables();
+        }
+      }
+
+      @Override
+      public boolean hasNext() {
+        return pos < sub.size();
+      }
+
+      @Override
+      public Type next() {
+        if (!hasNext()) {
+          throw new NoSuchElementException();
+        }
+        Type result = TypeSystemImpl.this.types.get(sub.get(pos));
+        pos++;
+        if (isTop) {
+          skipOverNonCreatables();
+        }
+        return result;
+      }
+
+      @Override
+      public void remove() {
+        throw new UnsupportedOperationException();
+      }
+      
+      private void skipOverNonCreatables() {
+        if (!hasNext()) {
+          return;
+        }
+        int typeCode = sub.get(pos);
+        while (! ll_isPrimitiveArrayType(typeCode) &&
+               ! casMetadata.creatableType[typeCode]) {
+          pos++;
+          if (!hasNext()) {
+            break;
+          }
+          typeCode = sub.get(pos);
+        }
+      }
+    };
+  }
+  
   public boolean directlySubsumes(int t1, int t2) {
     IntVector sub = this.tree.get(t1);
     return sub.contains(t2);
@@ -835,6 +923,7 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
    *          Subtype.
    * @return <code>true</code> iff <code>sub</code> inherits from <code>super</code>.
    */
+  @Override
   public boolean subsumes(Type superType, Type subType) {
     // assert(superType != null);
     // assert(subType != null);
@@ -844,6 +933,7 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
   /**
    * Get an array of the appropriate features for this type.
    */
+  @Override
   public int[] ll_getAppropriateFeatures(int type) {
     if (type < LEAST_TYPE_CODE || type > getNumberOfTypes()) {
       return null;
@@ -994,18 +1084,18 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
       // System.out.println("Size of type table > 0.");
       return 0;
     } // Add name of top type to symbol table.
-    this.top = this.typeNameST.set(name);
+    if (top != this.typeNameST.set(name)) throw new RuntimeException();
     // System.out.println("Size of name table is: " + typeNameST.size());
     // assert (typeNameST.size() == 1);
-    // System.out.println("Code of top type is: " + this.top);
+    // System.out.println("Code of top type is: " + top);
     // Create space for top type.
     newType();
     // Make top subsume itself.
-    addSubsubsumption(this.top, this.top);
-    this.types.add(new TypeImpl(name, this.top, this));
+    addSubsubsumption(top, top);
+    this.types.add(new TypeImpl(name, top, this));
     this.parents.add(LowLevelTypeSystem.UNKNOWN_TYPE_CODE);
     this.numCommittedTypes = this.types.size();
-    return this.top;
+    return top;
   }
 
   /**
@@ -1019,12 +1109,13 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
   }
 
   private boolean ll_isPrimitiveArrayType(int type) {
-    return type == this.floatArrayTypeCode || type == this.intArrayTypeCode
-        || type == this.booleanArrayTypeCode || type == this.shortArrayTypeCode
-        || type == this.byteArrayTypeCode || type == this.longArrayTypeCode
-        || type == this.doubleArrayTypeCode || type == this.stringArrayTypeCode;
+    return type == floatArrayTypeCode || type == intArrayTypeCode
+        || type == booleanArrayTypeCode || type == shortArrayTypeCode
+        || type == byteArrayTypeCode || type == longArrayTypeCode
+        || type == doubleArrayTypeCode || type == stringArrayTypeCode;
   }
 
+  @Override
   public boolean ll_subsumes(int superType, int type) {
     // Add range check.
     // assert (isType(superType));
@@ -1045,12 +1136,12 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
     // (this second relation because all we can generate are instances of
     // FSArray
     // and we must be able to assign them to xxx[] )
-    if (superType == this.fsArrayTypeCode) {
+    if (superType == fsArrayTypeCode) {
       return !ll_isPrimitiveArrayType(type) && ll_isArrayType(type);
     }
 
-    if (type == this.fsArrayTypeCode) {
-      return superType == this.top || superType == this.arrayBaseTypeCode
+    if (type == fsArrayTypeCode) {
+      return superType == top || superType == arrayBaseTypeCode
           || (!ll_isPrimitiveArrayType(superType) && ll_isArrayType(superType));
     }
 
@@ -1069,7 +1160,7 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
     } else if (isSubArray) {
       // If the subtype is an array, and the supertype is not, then the
       // supertype must be top, or the abstract array base.
-      return ((superType == this.top) || (superType == this.arrayBaseTypeCode));
+      return ((superType == top) || (superType == arrayBaseTypeCode));
     }
     return this.subsumes.get(superType).get(type);
   }
@@ -1139,6 +1230,7 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
   /**
    * @see org.apache.uima.cas.admin.TypeSystemMgr#commit()
    */
+  @Override
   public void commit() {
     if (this.locked) {
       return; // might be called multiple times, but only need to do once
@@ -1158,10 +1250,11 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
     // this.cas.commitTypeSystem();
     // }
   }
-
+  
   /**
    * @see org.apache.uima.cas.admin.TypeSystemMgr#isCommitted()
    */
+  @Override
   public boolean isCommitted() {
     return this.locked;
   }
@@ -1198,6 +1291,7 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
   /**
    * @see org.apache.uima.cas.admin.TypeSystemMgr#setFeatureFinal(org.apache.uima.cas.Type)
    */
+  @Override
   public void setFeatureFinal(Type type) {
     ((TypeImpl) type).setFeatureFinal();
   }
@@ -1205,6 +1299,7 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
   /**
    * @see org.apache.uima.cas.admin.TypeSystemMgr#setInheritanceFinal(org.apache.uima.cas.Type)
    */
+  @Override
   public void setInheritanceFinal(Type type) {
     ((TypeImpl) type).setInheritanceFinal();
   }
@@ -1212,6 +1307,7 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
   /**
    * @see org.apache.uima.cas.admin.TypeSystemMgr#addStringSubtype
    */
+  @Override
   public Type addStringSubtype(String typeName, String[] stringList) throws CASAdminException {
     // final int stringSetCode = this.stringSets.size();
     Type mother = this.stringType;
@@ -1245,6 +1341,7 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
    * 
    * @see org.apache.uima.cas.TypeSystem#getTypeNameSpace(java.lang.String)
    */
+  @Override
   public TypeNameSpace getTypeNameSpace(String name) {
     if (!TypeSystemUtils.isTypeNameSpaceName(name)) {
       return null;
@@ -1257,6 +1354,7 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
    * 
    * @see org.apache.uima.cas.impl.LowLevelTypeSystem#ll_getCodeForTypeName(java.lang.String)
    */
+  @Override
   public int ll_getCodeForTypeName(String typeName) {
     if (typeName == null) {
       throw new NullPointerException();
@@ -1269,10 +1367,12 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
    * 
    * @see org.apache.uima.cas.impl.LowLevelTypeSystem#ll_getCodeForType(org.apache.uima.cas.Type)
    */
+  @Override
   public int ll_getCodeForType(Type type) {
     return ((TypeImpl) type).getCode();
   }
 
+  @Override
   public int ll_getCodeForFeatureName(String featureName) {
     if (featureName == null) {
       throw new NullPointerException();
@@ -1283,10 +1383,12 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
     return this.featureMap.get(featureName);
   }
 
+  @Override
   public int ll_getCodeForFeature(Feature feature) {
     return ((FeatureImpl) feature).getCode();
   }
 
+  @Override
   public Type ll_getTypeForCode(int typeCode) {
     if (isType(typeCode)) {
       return this.types.get(typeCode);
@@ -1302,6 +1404,7 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
     return ((featureCode > UNKNOWN_FEATURE_CODE) && (featureCode <= getLargestFeatureCode()));
   }
 
+  @Override
   public Feature ll_getFeatureForCode(int featureCode) {
     if (isFeature(featureCode)) {
       return this.features.get(featureCode);
@@ -1309,22 +1412,27 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
     return null;
   }
 
+  @Override
   public int ll_getDomainType(int featureCode) {
     return intro(featureCode);
   }
 
+  @Override
   public int ll_getRangeType(int featureCode) {
     return range(featureCode);
   }
 
+  @Override
   public LowLevelTypeSystem getLowLevelTypeSystem() {
     return this;
   }
 
+  @Override
   public boolean ll_isStringSubtype(int type) {
     return this.stringSetMap.containsKey(type);
   }
 
+  @Override
   public boolean ll_isRefType(int typeCode) {
     final int typeClass = ll_getTypeClass(typeCode);
     switch (typeClass) {
@@ -1344,6 +1452,7 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
     }
   }
 
+  @Override
   public Type getArrayType(Type componentType) {
     final int arrayTypeCode = ll_getArrayType(ll_getCodeForType(componentType));
     if (arrayTypeCode == UNKNOWN_TYPE_CODE) {
@@ -1352,56 +1461,57 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
     return this.types.get(arrayTypeCode);
   }
 
+  @Override
   public final int ll_getTypeClass(int typeCode) {
-    if (typeCode == this.booleanTypeCode) {
+    if (typeCode == booleanTypeCode) {
       return LowLevelCAS.TYPE_CLASS_BOOLEAN;
     }
-    if (typeCode == this.byteTypeCode) {
+    if (typeCode == byteTypeCode) {
       return LowLevelCAS.TYPE_CLASS_BYTE;
     }
-    if (typeCode == this.shortTypeCode) {
+    if (typeCode == shortTypeCode) {
       return LowLevelCAS.TYPE_CLASS_SHORT;
     }
-    if (typeCode == this.intTypeCode) {
+    if (typeCode == intTypeCode) {
       return LowLevelCAS.TYPE_CLASS_INT;
     }
-    if (typeCode == this.floatTypeCode) {
+    if (typeCode == floatTypeCode) {
       return LowLevelCAS.TYPE_CLASS_FLOAT;
     }
-    if (typeCode == this.longTypeCode) {
+    if (typeCode == longTypeCode) {
       return LowLevelCAS.TYPE_CLASS_LONG;
     }
-    if (typeCode == this.doubleTypeCode) {
+    if (typeCode == doubleTypeCode) {
       return LowLevelCAS.TYPE_CLASS_DOUBLE;
     }
     // false if string type code not yet set up (during initialization)
     //   need this to avoid NPE in subsumes
-    if ((this.stringTypeCode != LowLevelTypeSystem.UNKNOWN_TYPE_CODE) &&
-          ll_subsumes(this.stringTypeCode, typeCode)) {
+    if ((stringTypeCode != LowLevelTypeSystem.UNKNOWN_TYPE_CODE) &&
+          ll_subsumes(stringTypeCode, typeCode)) {
       return LowLevelCAS.TYPE_CLASS_STRING;
     }
-    if (typeCode == this.booleanArrayTypeCode) {
+    if (typeCode == booleanArrayTypeCode) {
       return LowLevelCAS.TYPE_CLASS_BOOLEANARRAY;
     }
-    if (typeCode == this.byteArrayTypeCode) {
+    if (typeCode == byteArrayTypeCode) {
       return LowLevelCAS.TYPE_CLASS_BYTEARRAY;
     }
-    if (typeCode == this.shortArrayTypeCode) {
+    if (typeCode == shortArrayTypeCode) {
       return LowLevelCAS.TYPE_CLASS_SHORTARRAY;
     }
-    if (typeCode == this.intArrayTypeCode) {
+    if (typeCode == intArrayTypeCode) {
       return LowLevelCAS.TYPE_CLASS_INTARRAY;
     }
-    if (typeCode == this.floatArrayTypeCode) {
+    if (typeCode == floatArrayTypeCode) {
       return LowLevelCAS.TYPE_CLASS_FLOATARRAY;
     }
-    if (typeCode == this.longArrayTypeCode) {
+    if (typeCode == longArrayTypeCode) {
       return LowLevelCAS.TYPE_CLASS_LONGARRAY;
     }
-    if (typeCode == this.doubleArrayTypeCode) {
+    if (typeCode == doubleArrayTypeCode) {
       return LowLevelCAS.TYPE_CLASS_DOUBLEARRAY;
     }
-    if (typeCode == this.stringArrayTypeCode) {
+    if (typeCode == stringArrayTypeCode) {
       return LowLevelCAS.TYPE_CLASS_STRINGARRAY;
     }
     if (ll_isArrayType(typeCode)) {
@@ -1410,6 +1520,7 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
     return LowLevelCAS.TYPE_CLASS_FS;
   }
 
+  @Override
   public int ll_getArrayType(int componentTypeCode) {
     if (this.componentToArrayTypeMap.containsKey(componentTypeCode)) {
       return this.componentToArrayTypeMap.get(componentTypeCode);
@@ -1438,8 +1549,12 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
     TypeImpl arrayType = new TypeImpl(arrayTypeName, arrayTypeCode, this);
     this.types.add(arrayType);
     this.parents.add(motherCode);
-    if (!isCommitted())
+    if (!isCommitted()) {
       this.numCommittedTypes = this.types.size();
+    }
+//    if (null == this.arrayCodeToTypeMap.getReserve(arrayTypeCode)) {
+//      this.arrayCodeToTypeMap.put(arrayType);
+//    }
     this.arrayCodeToTypeMap.put(arrayTypeCode, arrayType);
 //    System.out.println("*** adding to arrayCodeToTypeMap: " + arrayType.getName() + ", committed=" + isCommitted());
     // For built-in arrays, we need to add the abstract base array as parent
@@ -1447,19 +1562,21 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
     // array are all over the place. Would be nice to just remove it.
     // Add an edge to the tree.
     if (!isCommitted() && motherCode != fsArrayTypeCode ) {
-      final int arrayBaseTypeCodeBeforeCommitted = this.arrayBaseTypeCode;
+      final int arrayBaseTypeCodeBeforeCommitted = arrayBaseTypeCode;
       (this.tree.get(arrayBaseTypeCodeBeforeCommitted)).add(arrayTypeCode);
       // Update subsumption relation.
-      updateSubsumption(arrayTypeCode, this.arrayBaseTypeCode);
+      updateSubsumption(arrayTypeCode, arrayBaseTypeCode);
     }
     return arrayTypeCode;
   }
 
+  @Override
   public boolean ll_isValidTypeCode(int typeCode) {
     return (this.typeNameST.getSymbol(typeCode) != null)
         || this.arrayToComponentTypeMap.containsKey(typeCode);
   }
 
+  @Override
   public boolean ll_isArrayType(int typeCode) {
 //    if (!ll_isValidTypeCode(typeCode)) {
 //      return false;
@@ -1467,6 +1584,7 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
     return this.arrayCodeToTypeMap.containsKey(typeCode);
   }
 
+  @Override
   public int ll_getComponentType(int arrayTypeCode) {
     if (ll_isArrayType(arrayTypeCode)) {
       return this.arrayToComponentTypeMap.get(arrayTypeCode);
@@ -1475,10 +1593,12 @@ public class TypeSystemImpl implements TypeSystemMgr, LowLevelTypeSystem {
   }
 
   /* note that subtypes of String are considered primitive */
+  @Override
   public boolean ll_isPrimitiveType(int typeCode) {
     return !ll_isRefType(typeCode);
   }
 
+  @Override
   public String[] ll_getStringSet(int typeCode) {
 //    if (!ll_isValidTypeCode(typeCode)) {
 //      return null;
