@@ -51,9 +51,9 @@ public class XCASDeserializer {
 
   private static class FSInfo {
 
-    private int addr;
+    final private int addr;
 
-    private IntVector indexRep;
+    final private IntVector indexRep;
 
     private FSInfo(int addr, IntVector indexRep) {
       super();
@@ -118,13 +118,13 @@ public class XCASDeserializer {
     private Locator locator;
 
     // The CAS we're filling.
-    private CASImpl cas;
+    final private CASImpl cas;
 
     // Store FSs with ID in a search tree (for later reference resolution).
-    private RedBlackTree<FSInfo> fsTree;
+    final private RedBlackTree<FSInfo> fsTree;
 
     // Store IDless FSs in a vector;
-    private List<FSInfo> idLess;
+    final private List<FSInfo> idLess;
 
     // What we expect next.
     private int state;
@@ -149,23 +149,23 @@ public class XCASDeserializer {
     private FSData currentOotsFs;
 
     // SofaFS type
-    private int sofaTypeCode;
+    final private int sofaTypeCode;
 
     // AnnotationBase type
-    private Type annotBaseType;
+    final private Type annotBaseType;
 
     // Store IndexRepositories in a vector;
-    private List<FSIndexRepository> indexRepositories;
+    final private List<FSIndexRepository> indexRepositories;
 
     // and Views too
-    private List<CAS> views;
+    final private List<CAS> views;
 
     // for processing v1.x format XCAS
     // map from sofa int values to id references
-    private IntVector sofaRefMap;
+    final  private IntVector sofaRefMap;
 
     // map incoming _indexed values
-    private IntVector indexMap;
+    final private IntVector indexMap;
 
     // working with initial view
     private int nextIndex;
@@ -313,7 +313,7 @@ public class XCASDeserializer {
       // Hang on address for setting content feature
       this.currentAddr = addr;
       int id = -1;
-      IntVector indexRep = new IntVector(); // empty means not indexed
+      IntVector indexRep = new IntVector(1); // empty means not indexed
       String attrName, attrValue;
       final int heapValue = cas.getHeapValue(addr);
       final Type type = cas.ll_getTypeSystem().ll_getTypeForCode(cas.ll_getFSRefType(addr));
@@ -764,7 +764,7 @@ public class XCASDeserializer {
       // "Resolving references for non-id data (" + idLess.size() + ").");
       // time = System.currentTimeMillis();
       for (int i = 0; i < idLess.size(); i++) {
-        finalizeFS((FSInfo) idLess.get(i));
+        finalizeFS(idLess.get(i));
       }
       // time = System.currentTimeMillis() - time;
       // System.out.println("Done in " + new TimeSpan(time));
@@ -777,29 +777,32 @@ public class XCASDeserializer {
         finalizeOutOfTypeSystemFeatures();
       }
 
-      for (int i = 0; i < views.size(); i++) {
-        ((CASImpl) views.get(i)).updateDocumentAnnotation();
+      for (CAS view : views) {
+        AutoCloseable ac = view.protectIndexes();
+        try {
+          ((CASImpl)view).updateDocumentAnnotation();
+        } finally {
+          try {
+            ac.close();
+          } catch (Exception e) {
+            assert(false);
+          }
+        }
       }
+//      for (int i = 0; i < views.size(); i++) {       
+//        ((CASImpl) views.get(i)).updateDocumentAnnotation();
+//      }
     }
 
     private void finalizeFS(FSInfo fsInfo) {
       final int addr = fsInfo.addr;
-      if (fsInfo.indexRep.size() >= 0) {
-        // Now add FS to all specified index repositories
-        for (int i = 0; i < fsInfo.indexRep.size(); i++) {
-          if (indexMap.size() == 1) {
-            ((FSIndexRepositoryImpl) indexRepositories.get(fsInfo.indexRep.get(i))).addFS(addr);
-          } else {
-            ((FSIndexRepositoryImpl) indexRepositories.get(indexMap.get(fsInfo.indexRep.get(i))))
-                    .addFS(addr);
-          }
-        }
-      }
       final int type = cas.getHeapValue(addr);
       if (cas.isArrayType(type)) {
         finalizeArray(type, addr, fsInfo);
+        finalizeAddToIndexes(fsInfo, addr);
         return;
       }
+      
       int[] feats = cas.getTypeSystemImpl().ll_getAppropriateFeatures(type);
       int feat;
       FSInfo fsValInfo;
@@ -807,7 +810,7 @@ public class XCASDeserializer {
         feat = feats[i];
         if (cas.ll_isRefType(ts.range(feats[i]))) {
           int featVal = cas.getFeatureValue(addr, feat);
-          fsValInfo = (FSInfo) fsTree.get(featVal);
+          fsValInfo = fsTree.get(featVal);
           if (fsValInfo == null) {
             cas.setFeatureValue(addr, feat, CASImpl.NULL);
             // this feature may be a ref to an out-of-typesystem FS.
@@ -829,8 +832,23 @@ public class XCASDeserializer {
           }
         }
       }
+      finalizeAddToIndexes(fsInfo, addr);  // must be done after above fixes the sofa refs
     }
 
+    private void finalizeAddToIndexes(final FSInfo fsInfo, final int addr) {
+      if (fsInfo.indexRep.size() >= 0) {
+        // Now add FS to all specified index repositories
+        for (int i = 0; i < fsInfo.indexRep.size(); i++) {
+          if (indexMap.size() == 1) {
+            ((FSIndexRepositoryImpl) indexRepositories.get(fsInfo.indexRep.get(i))).addFS(addr);
+          } else {
+            ((FSIndexRepositoryImpl) indexRepositories.get(indexMap.get(fsInfo.indexRep.get(i))))
+                    .addFS(addr);
+          }
+        }
+      }
+    }
+    
     private void finalizeArray(int type, int addr, FSInfo fsInfo) {
       if (!cas.isFSArrayType(type)) {
         // Nothing to do.
@@ -840,7 +858,7 @@ public class XCASDeserializer {
       FSInfo fsValInfo;
       for (int i = 0; i < size; i++) {
         int arrayVal = cas.getArrayValue(addr, i);
-        fsValInfo = (FSInfo) fsTree.get(arrayVal);
+        fsValInfo = fsTree.get(arrayVal);
         if (fsValInfo == null) {
           cas.setArrayValue(addr, i, CASImpl.NULL);
           // this element may be a ref to an out-of-typesystem FS.
@@ -878,7 +896,7 @@ public class XCASDeserializer {
           if (val >= 0) // negative numbers represent null and are left unchanged
           {
             // attempt to locate target in type system
-            FSInfo fsValInfo = (FSInfo) fsTree.get(val);
+            FSInfo fsValInfo = fsTree.get(val);
             if (fsValInfo != null) {
               entry.setValue(Integer.toString(fsValInfo.addr));
             } else
@@ -1067,9 +1085,9 @@ public class XCASDeserializer {
     }
   }
 
-  private TypeSystemImpl ts;
+  final private TypeSystemImpl ts;
 
-  private UimaContext uimaContext;
+  final private UimaContext uimaContext;
 
   // private HashMap featureMap; -APL
   // ///private int[] featureType;
