@@ -158,6 +158,7 @@ public class ResourceManager_impl implements ResourceManager {
   /**
    * Creates a new <code>ResourceManager_impl</code> with a custom ClassLoader to use for locating
    * resources.
+   * @param aClassLoader -
    */
   public ResourceManager_impl(ClassLoader aClassLoader) {
     mResourceMap = Collections.synchronizedMap(new HashMap<String, Object>());
@@ -190,8 +191,8 @@ public class ResourceManager_impl implements ResourceManager {
   * things including the Component Description Editor when
   * obtaining info from CustomResourceSpecifiers
   * https://issues.apache.org/jira/browse/UIMA-1722
-  * @param uimaCL
-  * @param resolveResource
+  * @param uimaCL -
+  * @param resolveResource -
   */
  public synchronized void setExtensionClassPath(UIMAClassLoader uimaCL, boolean resolveResource) {
    this.uimaCL = uimaCL;
@@ -450,7 +451,7 @@ public class ResourceManager_impl implements ResourceManager {
       String name = resources[i].getName();
 
       // check for existing resource registration under this name
-      ResourceRegistration registration = (ResourceRegistration) mInternalResourceRegistrationMap
+      ResourceRegistration registration = mInternalResourceRegistrationMap
               .get(name);
       if (registration == null) {
         registerResource(name, resources[i], aQualifiedContextName, aAdditionalParams);
@@ -476,7 +477,7 @@ public class ResourceManager_impl implements ResourceManager {
     // apply bindings
     ExternalResourceBinding[] bindings = aConfiguration.getExternalResourceBindings();
     for (int i = 0; i < bindings.length; i++) {
-      ResourceRegistration registration = (ResourceRegistration) mInternalResourceRegistrationMap
+      ResourceRegistration registration = mInternalResourceRegistrationMap
               .get(bindings[i].getResourceName());
       if (registration == null) {
         throw new ResourceInitializationException(
@@ -660,6 +661,11 @@ public class ResourceManager_impl implements ResourceManager {
    * @see org.apache.uima.resource.ResourceManager#getCasManager()
    */
   public CasManager getCasManager() {
+    //Optimization for case where mCasManager already created
+    // Some sync contention was observed - this makes it less.  UIMA-4012
+    if(mCasManager != null) {
+      return mCasManager;
+    }
     synchronized(casManagerMonitor) {
       if (mCasManager == null) {
         mCasManager = new CasManager_impl(this);
