@@ -281,6 +281,23 @@ public class CasCopier {
     mDestSofaFeatureCode = ((FeatureImpl)mDestSofaFeature).getCode();
     srcSofaTypeCode = originalSrcCasImpl.getTypeSystemImpl().sofaType.getCode();
     this.lenient = lenient;
+    
+    // the next is to support the style of use where
+    //   an instance of this copier is made, corresponding to two views in the same CAS
+    //   or corresponding to two views in different CASs
+    //   and then individual FeatureStructures are copied using copyFS(...)
+    
+    srcCasViewImpl = (CASImpl) originalSrcCas.getLowLevelCAS();
+    tgtCasViewImpl = (CASImpl) originalTgtCas.getLowLevelCAS();
+    
+    srcViewName = srcCasViewImpl.getViewName();
+    tgtViewName = tgtCasViewImpl.getViewName();
+    
+    if (srcViewName == null) {
+      isChangeViewName = (tgtViewName == null) ? false : true;
+    } else {
+      isChangeViewName = !srcViewName.equals(tgtViewName);
+    }
   }
   
 
@@ -343,7 +360,6 @@ public class CasCopier {
     Iterator<CAS> viewIterator = aSrcCas.getViewIterator();
     while (viewIterator.hasNext()) {
       CAS view = viewIterator.next();
-      copier.srcCasDocumentAnnotation = 0;  // each view needs to get this once
       copier.copyCasView(view, aCopySofa); 
 
     }
@@ -477,6 +493,8 @@ public class CasCopier {
       throw new UIMARuntimeException(UIMARuntimeException.UNSUPPORTED_CAS_COPY_TO_OR_FROM_BASE_CAS, null);
     }
     
+    srcCasDocumentAnnotation = 0;  // each view needs to get this once
+    
 //    mLowLevelDestCas = aTgtCasView.getLowLevelCAS();
 //    mLowLevelSrcCas = aSrcCasView.getLowLevelCAS();
     
@@ -587,6 +605,8 @@ public class CasCopier {
       tgtCasViewImpl = originalTgtCasImpl;
     }
     
+    // safety - insure DocumentAnnotation is tested.
+    srcCasDocumentAnnotation = 0;  
     return copyFs2Fs(aFS);
   }
   
@@ -641,6 +661,7 @@ public class CasCopier {
     // same Sofa ID in the target CAS. If it does not exist it will be created.
     if (srcTypeCode == srcSofaTypeCode) {
       String destSofaId = getDestSofaId(srcCasViewImpl.ll_getSofaID(aFS));
+      // note: not put into the mFsMap, because each view needs a separate copy
       return ((CASImpl)getOrCreateView(originalTgtCas, destSofaId)).getSofaRef();
     }
 
@@ -672,6 +693,7 @@ public class CasCopier {
           }
         }
       }
+      // note note put into mFsMap, because each view needs a separate copy
       return destDocAnnot;
     }
 
@@ -777,6 +799,17 @@ public class CasCopier {
     }
   }
   
+  /**
+   * Note: if lenient is in effect, this method will return false for
+   * FSs which are not copied because the target doesn't have that type.
+   * It also returns false for sofa FSs and the documentAnnotation FS.
+   * @param aFS a feature structure
+   * @return true if the given FS has already been copied using this CasCopier.
+   */
+  public boolean alreadyCopied(FeatureStructure aFS) {
+    return alreadyCopied(((FeatureStructureImpl)aFS).getAddress());
+  }
+  
   /**
    * Note: if lenient is in effect, this method will return false for
    * FSs which are not copied because the target doesn't have that type.
@@ -799,32 +832,32 @@ public class CasCopier {
     
     int srcTypeCode = srcCasViewImpl.getTypeCode(srcFS);
     
-    if (srcTypeCode == srcTsi.stringArrayTypeCode) {
-      final int tgtFS = tgtCasViewImpl.ll_createArray(tgtTsi.stringArrayTypeCode, len);
+    if (srcTypeCode == TypeSystemImpl.stringArrayTypeCode) {
+      final int tgtFS = tgtCasViewImpl.ll_createArray(TypeSystemImpl.stringArrayTypeCode, len);
       for (int i = 0; i < len; i++) {
         tgtCasViewImpl.ll_setStringArrayValue(tgtFS, i, srcCasViewImpl.ll_getStringArrayValue(srcFS, i));
       }
       return tgtFS;
     }
     
-    if (srcTypeCode == srcTsi.intArrayTypeCode) {     
-      final int tgtFS = tgtCasViewImpl.ll_createArray(tgtTsi.intArrayTypeCode, len);
+    if (srcTypeCode == TypeSystemImpl.intArrayTypeCode) {     
+      final int tgtFS = tgtCasViewImpl.ll_createArray(TypeSystemImpl.intArrayTypeCode, len);
       for (int i = 0; i < len; i++) {
         tgtCasViewImpl.ll_setIntArrayValue(tgtFS, i,  srcCasViewImpl.ll_getIntArrayValue(srcFS,  i));
       }
       return tgtFS;
     }
 
-    if (srcTypeCode == srcTsi.floatArrayTypeCode) {     
-      final int tgtFS = tgtCasViewImpl.ll_createArray(tgtTsi.floatArrayTypeCode, len);
+    if (srcTypeCode == TypeSystemImpl.floatArrayTypeCode) {     
+      final int tgtFS = tgtCasViewImpl.ll_createArray(TypeSystemImpl.floatArrayTypeCode, len);
       for (int i = 0; i < len; i++) {
         tgtCasViewImpl.ll_setFloatArrayValue(tgtFS, i,  srcCasViewImpl.ll_getFloatArrayValue(srcFS, i));
       }
       return tgtFS;
     }
 
-    if (srcTypeCode == srcTsi.fsArrayTypeCode) {     
-      final int tgtFS = tgtCasViewImpl.ll_createArray(tgtTsi.fsArrayTypeCode, len);
+    if (srcTypeCode == TypeSystemImpl.fsArrayTypeCode) {     
+      final int tgtFS = tgtCasViewImpl.ll_createArray(TypeSystemImpl.fsArrayTypeCode, len);
       for (int i = 0; i < len; i++) {
         int srcItem = srcCasViewImpl.ll_getRefArrayValue(srcFS, i);
         tgtCasViewImpl.ll_setRefArrayValue(tgtFS, i, (srcItem == 0) ? 0 : copyFsInner(srcItem));
@@ -832,7 +865,7 @@ public class CasCopier {
       return tgtFS;
     }
 
-    if (srcTypeCode == srcTsi.byteArrayTypeCode) {
+    if (srcTypeCode == TypeSystemImpl.byteArrayTypeCode) {
       final int tgtFS = tgtCasViewImpl.ll_createByteArray(len);
       for (int i = 0; i < len; i++) {
         tgtCasViewImpl.ll_setByteArrayValue(tgtFS,  i, srcCasViewImpl.ll_getByteArrayValue(srcFS, i));
@@ -840,7 +873,7 @@ public class CasCopier {
       return tgtFS;      
     }
     
-    if (srcTypeCode == srcTsi.shortArrayTypeCode) {
+    if (srcTypeCode == TypeSystemImpl.shortArrayTypeCode) {
       final int tgtFS = tgtCasViewImpl.ll_createShortArray(len);
       for (int i = 0; i < len; i++) {
         tgtCasViewImpl.ll_setShortArrayValue(tgtFS,  i, srcCasViewImpl.ll_getShortArrayValue(srcFS, i));
@@ -848,7 +881,7 @@ public class CasCopier {
       return tgtFS;      
     }
 
-    if (srcTypeCode == srcTsi.longArrayTypeCode) {
+    if (srcTypeCode == TypeSystemImpl.longArrayTypeCode) {
       final int tgtFS = tgtCasViewImpl.ll_createLongArray(len);
       for (int i = 0; i < len; i++) {
         tgtCasViewImpl.ll_setLongArrayValue(tgtFS,  i, srcCasViewImpl.ll_getLongArrayValue(srcFS, i));
@@ -856,7 +889,7 @@ public class CasCopier {
       return tgtFS;      
     }
     
-    if (srcTypeCode == srcTsi.doubleArrayTypeCode) {
+    if (srcTypeCode == TypeSystemImpl.doubleArrayTypeCode) {
       final int tgtFS = tgtCasViewImpl.ll_createDoubleArray(len);
       for (int i = 0; i < len; i++) {
         tgtCasViewImpl.ll_setDoubleArrayValue(tgtFS,  i, srcCasViewImpl.ll_getDoubleArrayValue(srcFS, i));
@@ -864,7 +897,7 @@ public class CasCopier {
       return tgtFS;      
     }
 
-    if (srcTypeCode == srcTsi.booleanArrayTypeCode) {
+    if (srcTypeCode == TypeSystemImpl.booleanArrayTypeCode) {
       final int tgtFS = tgtCasViewImpl.ll_createBooleanArray(len);
       for (int i = 0; i < len; i++) {
         tgtCasViewImpl.ll_setBooleanArrayValue(tgtFS,  i, srcCasViewImpl.ll_getBooleanArrayValue(srcFS, i));
