@@ -275,7 +275,7 @@ public class XMLParser_impl implements XMLParser {
    * 
    * @return an <code>XMLizable</code> object constructed from the DOM element
    * 
-   * @throws InvaidXMLException
+   * @throws InvalidXMLException
    *           if the XML element does not specify a valid object
    */
   public XMLizable buildObject(Element aElement) throws InvalidXMLException {
@@ -290,7 +290,7 @@ public class XMLParser_impl implements XMLParser {
    * 
    * @return an <code>XMLizable</code> object constructed from the DOM element
    * 
-   * @throws InvaidXMLException
+   * @throws InvalidXMLException
    *           if the XML element does not specify a valid object
    */
   public XMLizable buildObject(Element aElement, ParsingOptions aOptions)
@@ -305,7 +305,7 @@ public class XMLParser_impl implements XMLParser {
     // resolve the class name and instantiate the class
     XMLizable object;
     try {
-      object = (XMLizable) cls.newInstance();
+      object = cls.newInstance();
     } catch (Exception e) {
       throw new UIMA_IllegalStateException(
               UIMA_IllegalStateException.COULD_NOT_INSTANTIATE_XMLIZABLE, new Object[] { cls
@@ -351,7 +351,7 @@ public class XMLParser_impl implements XMLParser {
     // resolve the class name and instantiate the class
     XMLizable object;
     try {
-      object = (XMLizable) cls.newInstance();
+      object = cls.newInstance();
     } catch (Exception e) {
       throw new UIMA_IllegalStateException(
               UIMA_IllegalStateException.COULD_NOT_INSTANTIATE_XMLIZABLE, new Object[] { cls
