@@ -28,9 +28,7 @@ import java.io.FileInputStream;
 import java.io.FileOutputStream;
 import java.io.FileReader;
 import java.io.IOException;
-import java.io.InputStreamReader;
 import java.io.OutputStreamWriter;
-import java.nio.CharBuffer;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.List;
@@ -581,11 +579,12 @@ public class SerDesTest4 extends TestCase {
   private static final String[] stringValues = {
     "abc", "abcdef", null, "", "ghijklm", "a", "b"
   };
+  
   private String randomString(Random r) {
     int i = r.nextInt(7);
-    if (i >= 7) {
-      System.out.println("debug");
-    }
+//    if (i >= 7) {
+//      System.out.println("debug");
+//    }
     return stringValues[i];
   }
 
@@ -698,7 +697,7 @@ public class SerDesTest4 extends TestCase {
     fs = newAkof(fsl);
     fs.setBooleanValue(akofBoolean, false);
     fs.setByteValue(akofByte, Byte.MAX_VALUE);
-    fs.setShortValue(akofShort, (short) Short.MAX_VALUE);
+    fs.setShortValue(akofShort, Short.MAX_VALUE);
     fs.setIntValue(akofInt, Integer.MAX_VALUE);
     fs.setFloatValue(akofFloat, Float.MAX_VALUE);
     fs.setLongValue(akofLong, Long.MAX_VALUE);
@@ -709,7 +708,7 @@ public class SerDesTest4 extends TestCase {
 
     fs = newAkof(fsl);
     fs.setByteValue(akofByte, Byte.MIN_VALUE);
-    fs.setShortValue(akofShort, (short) Short.MIN_VALUE);
+    fs.setShortValue(akofShort, Short.MIN_VALUE);
     fs.setIntValue(akofInt, Integer.MIN_VALUE);
     fs.setFloatValue(akofFloat, Float.MIN_VALUE);
     fs.setLongValue(akofLong, Long.MIN_VALUE);
