@@ -57,6 +57,8 @@ import org.apache.uima.cas.impl.XmiSerializationSharedData.OotsElementData;
 import org.apache.uima.cas.impl.XmiSerializationSharedData.XmiArrayElement;
 import org.apache.uima.cas.text.AnnotationFS;
 import org.apache.uima.cas_data.impl.CasComparer;
+import org.apache.uima.internal.util.MultiThreadUtils;
+import org.apache.uima.internal.util.MultiThreadUtils.ThreadM;
 import org.apache.uima.internal.util.XmlAttribute;
 import org.apache.uima.internal.util.XmlElementNameAndContents;
 import org.apache.uima.resource.metadata.FsIndexDescription;
@@ -259,16 +261,20 @@ public class XmiCasDeserializerTest extends TestCase {
   	
 		public void run() {
 			try {
-				serialize(cas, null);
+			  while (true) {
+  			  if (!MultiThreadUtils.wait4go((ThreadM) Thread.currentThread())) {
+  			    break;
+  			  }
+  
+  			  serialize(cas, null);
+			  }
 //				serialize(cas, null);
 //				serialize(cas, null);
 //				serialize(cas, null);
 			} catch (IOException e) {
-			
-				e.printStackTrace();
+			  throw new RuntimeException(e);
 			} catch (SAXException e) {
-				
-				e.printStackTrace();
+        throw new RuntimeException(e);				
 			}
 		}
   }
@@ -301,23 +307,29 @@ public class XmiCasDeserializerTest extends TestCase {
     	CasCopier.copyCas(cas, cases[i], true);
     }
     
-    // start n threads, serializing as XMI   
+    // start n threads, serializing as XMI
+    MultiThreadUtils.ThreadM [] threads = new MultiThreadUtils.ThreadM[MAX_THREADS];
+    for (int i = 0; i < MAX_THREADS; i++) {
+      threads[i] = new MultiThreadUtils.ThreadM(new DoSerialize(cases[i]));
+      threads[i].start();
+    }
+    MultiThreadUtils.waitForAllReady(threads);
     
     for (int i = 0; i < threadsToUse.length; i++) {
-    	Thread [] threads = new Thread[MAX_THREADS];
+      MultiThreadUtils.ThreadM[] sliceOfThreads = new MultiThreadUtils.ThreadM[threadsToUse[i]];
+      System.arraycopy(threads, 0, sliceOfThreads, 0, threadsToUse[i]);
+      
     	long startTime = System.currentTimeMillis();
-    	for (int ti = 0; ti < threadsToUse[i]; ti++) {
-    		threads[ti] = new Thread(new DoSerialize(cases[ti]));
-    		
-    		threads[ti].start();
-    	}
-    	for (int ti = 0; ti < threadsToUse[i]; ti++) {
-    		threads[ti].join();
-    		//System.out.print(" "+ ti);
-    	}
+    	
+    	MultiThreadUtils.kickOffThreads(sliceOfThreads);
+    	
+    	MultiThreadUtils.waitForAllReady(sliceOfThreads);
+    	
     	System.out.println("\nNumber of threads serializing: " + threadsToUse[i] + 
     			               "  Normalized millisecs (should be close to the same): " + (System.currentTimeMillis() - startTime) / threadsToUse[i]);
     }
+    
+    MultiThreadUtils.terminateThreads(threads);
   }
 
   public void testDeltaCasIndexExistingFsInView() throws Exception {
@@ -883,7 +895,7 @@ public class XmiCasDeserializerTest extends TestCase {
     deserialize(newSerCasMerged, cas, serSharedData, false, -1);
     
     //check covered text of annotations
-    FSIterator iter = cas.getAnnotationIndex().iterator();
+    FSIterator<AnnotationFS> iter = cas.getAnnotationIndex().iterator();
     while (iter.hasNext()) {
       AnnotationFS annot = (AnnotationFS)iter.next();
       assertEquals(cas.getDocumentText().substring(
@@ -893,7 +905,7 @@ public class XmiCasDeserializerTest extends TestCase {
     iter = cas.getAnnotationIndex(ownerType).iterator();
     while (iter.hasNext()) {
       AnnotationFS
-      annot = (AnnotationFS)iter.next();
+      annot = iter.next();
       String componentId = annot.getStringValue(componentIdFeat);
       if ("XCasDeserializerTest".equals(componentId)) {
         FeatureStructure targetRelArgs = annot.getFeatureValue(argsFeat);
@@ -968,7 +980,7 @@ public class XmiCasDeserializerTest extends TestCase {
 	  
 	  //create Marker, add/modify fs and serialize in delta xmi format.
 	  Marker marker = cas2.createMarker();
-	  FSIndex cas2tIndex = cas2.getAnnotationIndex();
+	  FSIndex<AnnotationFS> cas2tIndex = cas2.getAnnotationIndex();
 	  
 	  //create an annotation and add to index
 	  AnnotationFS cas2newAnnot = cas2.createAnnotation(cas2.getAnnotationType(), 6, 8);
@@ -976,7 +988,7 @@ public class XmiCasDeserializerTest extends TestCase {
 	  assertTrue(cas2tIndex.size() == 4); // prev annots and this new one
 	  
 	  //modify an existing annotation
-	  Iterator<FeatureStructure> tIndexIter = cas2tIndex.iterator();
+	  Iterator<AnnotationFS> tIndexIter = cas2tIndex.iterator();
 	  AnnotationFS docAnnot = (AnnotationFS) tIndexIter.next(); //doc annot
 	  //delete from index
 	  AnnotationFS delAnnot = (AnnotationFS) tIndexIter.next(); //annot
@@ -1030,7 +1042,7 @@ public class XmiCasDeserializerTest extends TestCase {
       
       //create Marker, add/modify fs and serialize in delta xmi format.
       Marker marker = cas2.createMarker();
-      FSIndex cas2tIndex = cas2.getAnnotationIndex();
+      FSIndex<AnnotationFS> cas2tIndex = cas2.getAnnotationIndex();
       
       //create an annotation and add to index
       AnnotationFS cas2newAnnot = cas2.createAnnotation(cas2.getAnnotationType(), 6, 8);
@@ -1038,7 +1050,7 @@ public class XmiCasDeserializerTest extends TestCase {
       assertTrue(cas2tIndex.size() == 4); // prev annots and this new one
       
       //modify language feature
-      Iterator<FeatureStructure> tIndexIter = cas2tIndex.iterator();
+      Iterator<AnnotationFS> tIndexIter = cas2tIndex.iterator();
       AnnotationFS docAnnot = (AnnotationFS) tIndexIter.next();
       Feature languageF = cas2.getDocumentAnnotation().getType().getFeatureByBaseName(CAS.FEATURE_BASE_NAME_LANGUAGE);
       docAnnot.setStringValue(languageF, "en");
@@ -1136,7 +1148,7 @@ public class XmiCasDeserializerTest extends TestCase {
 	      
 	      //create Marker, add/modify fs and serialize in delta xmi format.
 	      Marker marker = cas2.createMarker();
-	      FSIndex cas2tIndex = cas2.getAnnotationIndex();
+	      FSIndex<AnnotationFS> cas2tIndex = cas2.getAnnotationIndex();
 	      
 	      // serialize cas2 in delta format 
 	      String deltaxml1 = serialize(cas2, sharedData2, marker);
@@ -1167,7 +1179,7 @@ public class XmiCasDeserializerTest extends TestCase {
       cas1.getIndexRepository().addFS(anAnnot1);
       AnnotationFS anAnnot2 = cas1.createAnnotation(cas1.getAnnotationType(), 5, 10);
       cas1.getIndexRepository().addFS(anAnnot2);
-      FSIndex tIndex = cas1.getAnnotationIndex();
+      FSIndex<AnnotationFS> tIndex = cas1.getAnnotationIndex();
       assertTrue(tIndex.size() == 3); //doc annot plus 2 annots
       
       //serialize complete  
@@ -1182,7 +1194,7 @@ public class XmiCasDeserializerTest extends TestCase {
       
       //create Marker, add/modify fs and serialize in delta xmi format.
       Marker marker = cas2.createMarker();
-      FSIndex cas2tIndex = cas2.getAnnotationIndex();
+      FSIndex<AnnotationFS> cas2tIndex = cas2.getAnnotationIndex();
       
       //create an annotation and add to index
       AnnotationFS cas2newAnnot = cas2.createAnnotation(cas2.getAnnotationType(), 6, 8);
@@ -1190,7 +1202,7 @@ public class XmiCasDeserializerTest extends TestCase {
       assertTrue(cas2tIndex.size() == 4); // prev annots and this new one
       
       //modify language feature
-      Iterator<FeatureStructure> tIndexIter = cas2tIndex.iterator();
+      Iterator<AnnotationFS> tIndexIter = cas2tIndex.iterator();
       AnnotationFS docAnnot = (AnnotationFS) tIndexIter.next();
       
       //delete annotation from index    
@@ -1264,7 +1276,7 @@ public class XmiCasDeserializerTest extends TestCase {
       cas1.getIndexRepository().addFS(anAnnot3);
       AnnotationFS anAnnot4 = cas1.createAnnotation(cas1.getAnnotationType(), 15, 30);
       cas1.getIndexRepository().addFS(anAnnot4);
-      FSIndex tIndex = cas1.getAnnotationIndex();
+      FSIndex<AnnotationFS> tIndex = cas1.getAnnotationIndex();
       assertTrue(tIndex.size() == 5); //doc annot plus 4 annots
       
       FeatureStructure entityFS = cas1.createFS(entityType);
@@ -1318,7 +1330,7 @@ public class XmiCasDeserializerTest extends TestCase {
       //=======================================================================
       //create Marker, add/modify fs and serialize in delta xmi format.
       Marker marker = cas2.createMarker();
-      FSIndex cas2tIndex = cas2.getAnnotationIndex();
+      FSIndex<AnnotationFS> cas2tIndex = cas2.getAnnotationIndex();
       CAS cas2preexistingView = cas2.getView("preexistingView");
       FSIndex cas2personIndex = cas2preexistingView.getAnnotationIndex(personType);
       FSIndex cas2orgIndex = cas2preexistingView.getAnnotationIndex(orgType);
@@ -1339,7 +1351,7 @@ public class XmiCasDeserializerTest extends TestCase {
       assertTrue(cas2view1Index.size() == 2); //document annot and this annot
       
       //modify an existing annotation
-      Iterator<FeatureStructure> tIndexIter = cas2tIndex.iterator();
+      Iterator<AnnotationFS> tIndexIter = cas2tIndex.iterator();
       AnnotationFS docAnnot = (AnnotationFS) tIndexIter.next(); //doc annot
       AnnotationFS modAnnot1 = (AnnotationFS) tIndexIter.next();
       AnnotationFS delAnnot = (AnnotationFS)  tIndexIter.next();
