@@ -139,7 +139,6 @@ public class JCasTest extends TestCase {
 		try {
 			// jcasCasMisMatch(CASTestSetup.BAD_MISSING_FEATURE_IN_CAS, CASException.JCAS_INIT_ERROR);
 			CAS localCas;
-			TypeSystem ts;
 			JCas localJcas = null;
 			boolean errFound = false;
 			try {
@@ -184,7 +183,6 @@ public class JCasTest extends TestCase {
 	public void jcasCasMisMatch(int testId, String expectedErr) throws Exception {
 		try {
 			CAS localCas;
-			TypeSystem ts;
 			JCas localJcas;
 			boolean errFound = false;
 			try {
@@ -347,10 +345,22 @@ public class JCasTest extends TestCase {
 			FSIndex fsi1 = jfsi.getIndex("all", Root.type);
 			FSIterator fsit1 = fsi1.iterator();
 			assertTrue(fsit1.isValid());
-			assertTrue(r2 == fsit1.get());
-			fsit1.moveToNext();
-			assertTrue(fsit1.isValid());
-			assertTrue(r1 == fsit1.get());
+			Root[] fetched = new Root[2];
+			
+			fetched[0] = (Root) fsit1.get();
+      fsit1.moveToNext();
+      assertTrue(fsit1.isValid());
+
+      fetched[1] = (Root) fsit1.get();
+      assertTrue(fsit1.isValid());
+      fsit1.moveToNext();
+      assertFalse(fsit1.isValid());
+			
+      // is bag index, order may be arbitrary
+			assertTrue((fetched[0] == r1 && fetched[1] == r2) ||
+			           (fetched[1] == r1 && fetched[0] == r2));
+			
+			
 			/*
        * while (fsit1.isValid()) { System.out.println("Iterator getting: " +
        * fsit1.get().toString()); fsit1.moveToNext(); }
